"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Lint = require("tslint");
/* tslint:disable:max-classes-per-file */
const RULE_DESCRIPTION = `Trailing comma is required on interface properties.`;
class Rule extends Lint.Rules.AbstractRule {
    apply(sourceFile) {
        return this.applyWithWalker(new TrailingCommaInterfaceWalker(sourceFile, this.getOptions()));
    }
}
Rule.metadata = {
    description: RULE_DESCRIPTION,
    options: null,
    optionsDescription: `Not configurable.`,
    rationale: `Our team prefers trailing commas over trailing semicolons.`,
    ruleName: `trailing-comma-interface`,
    type: 'typescript',
    typescriptOnly: true,
};
exports.Rule = Rule;
class TrailingCommaInterfaceWalker extends Lint.RuleWalker {
    visitInterfaceDeclaration(node) {
        node.members
            .filter((propertySignature) => propertySignature.getText().slice(-1) !== `,`)
            .forEach((propertySignature) => {
            const lastChar = propertySignature.getText().slice(-1);
            const lastCharPosition = propertySignature.getStart() + propertySignature.getWidth() - 1;
            const replacementText = (lastChar === `;`) ? `,` : `${lastChar},`;
            const fixer = new Lint.Replacement(lastCharPosition, 1, replacementText);
            this.addFailureAt(lastCharPosition, 1, RULE_DESCRIPTION, fixer);
        });
    }
}
//# sourceMappingURL=trailingCommaInterfaceRule.js.map