import * as Lint from 'tslint';
import * as ts from 'typescript';

/* tslint:disable:max-classes-per-file */

const RULE_DESCRIPTION = `Trailing comma is required on interface properties.`;

export class Rule extends Lint.Rules.AbstractRule {
    public static metadata: Lint.IRuleMetadata = {
        description: RULE_DESCRIPTION,
        options: null,
        optionsDescription: `Not configurable.`,
        rationale: `Our team prefers trailing commas over trailing semicolons.`,
        ruleName: `trailing-comma-interface`,
        type: 'typescript',
        typescriptOnly: true,
    };

    public apply(sourceFile: ts.SourceFile): Lint.RuleFailure[] {
        return this.applyWithWalker(new TrailingCommaInterfaceWalker(sourceFile, this.getOptions()));
    }
}

class TrailingCommaInterfaceWalker extends Lint.RuleWalker {
    public visitInterfaceDeclaration(node: ts.InterfaceDeclaration) {
        node.members
            .filter((propertySignature) => propertySignature.getText().slice(-1) !== `,`)
            .forEach((propertySignature) => {
                const lastChar = propertySignature.getText().slice(-1);
                const lastCharPosition = propertySignature.getStart() + propertySignature.getWidth() - 1;
                const replacementText = (lastChar === `;`) ? `,` : `${lastChar},`;
                const fixer = new Lint.Replacement(lastCharPosition, 1, replacementText);
                this.addFailureAt(lastCharPosition, 1, RULE_DESCRIPTION, fixer);
            });
    }
}
